;******************************************************************************
;   This file is a basic code template for code generation on the             *
;   PIC12F1822. This file contains the basic code building blocks to build    *
;   upon.                                                                     *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on             *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:        Emergency Radiation Alarm.asm                           *
;    Date:           April 23, 2013                                           *
;    File Version:                                                            *
;                                                                             *
;    Author:        Ron Newton                                                *
;    Company:                                                                 *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: P12F1822.INC                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes:  Alarm can be change using line 132                               *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Revision History:                                                        *
;                                                                             *
;******************************************************************************


	list		p=12F1822      ; list directive to define processor
	#include	<p12F1822.inc> ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;------------------------------------------------------------------------------    

    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_OFF & _BORV_19 & _LVP_OFF

;------------------------------------------------------------------------------
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0-15.  Each Bank may contain 
; Special Function Registers, General Purpose Registers, and Access RAM 
;
;------------------------------------------------------------------------------

    CBLOCK 0x20 ; Define GPR variable register locations
    
;***** VARIABLE DEFINITIONS
w_temp       	 	
status_temp 	 	
SECOND         		
SECOND_TEN     		
MINUTE         		
DELAY		   		
COUNT		   		
COUNTER1       	
COUNTER2	  	
COUNTER3	   		
COUNTER4
COUNTER5
FRACTION			
ACCcLO				
ACCcMID			    
ACCcHI		    	
ACCaLO			    
ACCaMID		    	
ACCaHI			    
ACCbLO		        
ACCbMID			    
ACCbHI			    
temp		        
ACCdLO		        
ACCdMID			    
ACCdHI
FLAG			    

    ENDC

SAMPLE1        EQU    0x7D        ; Sample user registers (common RAM)
SAMPLE2        EQU    0x7E        ; 
SAMPLE3        EQU    0x7F        ; 

#DEFINE		IN	   	PORTA, 4     ;A/D IN PIN 3
#DEFINE		OUT     PORTA, 5 	;ALARM OUT PIN 2

msb_bit equ     7               ;define for bit 7
lsb_bit equ     0               ;define for bit 0

;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

    ORG     0x0000            ; processor reset vector
    GOTO    main              ; When using debug header, first inst.
                              ; may be passed over by ICD2.  

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------
		ORG     	0x004             ; interrupt vector location
		NOP							  ; FOR TESTING
		movwf   	w_temp            ; save off current W register contents
		movf		STATUS,w          ; move status register into W register
 		MOVWF		status_temp
		NOP								;FOR TESTING
		BANKSEL	PORTA					;CHANGE BANK
CLOCK	  DECFSZ	FRACTION,1		   ;DECREASE FRACTION
		  GOTO		INT1
		  MOVLW		.124			   ;RESET FRACTION
		  MOVWF		FRACTION		   ;LOAD 
		  DECFSZ	SECOND,1		   ;DECREASING SECOND
		  GOTO		INT1			   ;NOT ZERO JUMP
		  BSF		FLAG,0				;SET FLAG TO RE-MEASURE
		  MOVLW		.60					;LOAD SECOND
		  MOVWF		SECOND			    ;RESET SECOND
		  MOVLW		.100				;CHECK TO SEE HOW MANY COUNTS  set the alarm here
		  SUBWF		COUNT,0				;CHECK IF ABOVE ALARM LIMIT
		  BTFSS		3,0
		  GOTO		INT1			    ;NOT ABOVE JUMP
		  CALL		ALARM				;ABOVE PRESET
		  CLRF		COUNT				;CLEAR COUNT

INT1	BCF		INTCON,TMR0IF
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt
	

; these first 4 instructions are not required if the internal oscillator is not used
main
		BANKSEL		OSCCON			;CHANGE BANK		
		MOVLW		B'11110000'		;WORD = 1 16 MHZ
		MOVWF		OSCCON			;load
		BANKSEL		PORTA			;CHANGE BANK
		CLRF 		PORTA 			;Init PORTA
		MOVLW 		0Fh 			;Set GP<2:0> to
		BANKSEL		LATA			;CHANGE BANK
		CLRF		LATA			;CLEAR LATA
		BANKSEL		ANSELA			;CHANGE BANK
		MOVLW		B'00010000'		;LOAD ANALOG PIN
		MOVWF		ANSELA			;LOAD
	    BANKSEL		TRISA			;CHANGE BANK
		MOVLW		B'0011111'     	;PROGRAM PORTA PORTS
        MOVWF		TRISA 			;load
        BANKSEL		OPTION_REG		;CHANGE BANK
        MOVLW		B'10000111'     ;SET PRESCALER TO 16 TMRO
        MOVWF		OPTION_REG		;LOAD
		BANKSEL		ADCON0			;CHANGE BANK
		MOVLW		B'10001101'		;A/D ON 
		MOVWF		ADCON0			;LOAD
		BANKSEL		ADCON1			;CHANGE BANK
		MOVLW		B'10100011'		;RIGHT JUSTIFIED INTERNAL VOLTAGE REF
		MOVWF		ADCON1			;LOAD
		BANKSEL		FVRCON			;CHANGE BANK
		MOVLW		B'10000001'		;VOLTAGE REF ON  1.024 VOLTS
		MOVWF		FVRCON			;LOAD
		BANKSEL		INTCON			;CHANGE BANK
		MOVLW		B'10100000'		;ENABLE TMRO
		MOVWF		INTCON			;LOAD
		BANKSEL		PORTA			;CHANGE BANK
	    BSF			OUT				;SET OUT HIGH (SPEAKER OFF)
		MOVLW		.31				;ADJUST TMRO
		MOVWF		FRACTION		;LOAD
		MOVLW		.60				;LOAD SECONDS FOR 1 MINUTE
		MOVWF		SECOND			;LOAD
	    CLRF		COUNT			;CLEAR COUNT
;***************************************************************
TESTING	

;**************************************************************
HERE      CALL		ALARM				;TESTING BATTERY

WAIT	  CLRF		ACCbLO				;CLEARING REGISTERS
		  CLRF		ACCbMID
		  CLRF		ACCbHI
		  CLRF		ACCcHI
		  CLRF		ACCcMID
		  CLRF		ACCcLO
		  CLRF		ACCaHI
		  CLRF		ACCaMID
		  CLRF		ACCaLO
		  BCF		FLAG,0
		  MOVLW		.4					;SETTING FOR 1024 COUNTS
		  MOVWF		COUNTER2			;LOAD
W1		  NOP							;NECESSARY FOR BANKSEL
		  BANKSEL	ADCON0				;CHANGE BANK
		  BSF		ADCON0,ADGO			;TURN ON A/D
		  BTFSC		ADCON0,ADGO			;WAIT UNTIL FINISHED
		  GOTO		$-1

		  BANKSEL	ADRESH				;CHANGE BANK
		  MOVF		ADRESH,0			;UNLOAD ADRESH
		  BANKSEL	PORTA				;CHANGE BANK
		  ADDWF		ACCcMID,1			;LOAD INTO ACCcMID
		  BTFSC		3,0					;CHECK FOR CARRY
		  INCF		ACCcHI,1			;CARRY INCREASE ACCcHI
		  BANKSEL	ADRESL				;CHANGE BANK
		  MOVF		ADRESL,0			;UNLOAD ADRESL
		  BANKSEL	PORTA				;CHANGE BANK
		  ADDWF		ACCcLO,1			;LOAD INTO ACCcLO
		  BTFSS		3,0					;CHECK FOR CARRY
		  GOTO		W2					;JUMP
		  MOVLW		.1					;CARRY INCREASE ACCcMID
		  ADDWF		ACCcMID,1
		  BTFSC		3,0
		  INCF		ACCcHI,1			;CHECK FOR CARRY INCREASE ACCcHI

W2		  DECFSZ	COUNTER1,1			;COUNTING UP TO 256
		  GOTO		W1					;JUMP
		  DECFSZ	COUNTER2,1			;COUNTING UP 4 TIMES FOR 1024
		  GOTO		W1					;JUMP
		  MOVLW		.4					;LOAD 4 FOR DIVISION OF 1024
		  MOVWF		ACCaMID				;LOAD
		  CALL		Div24				;CALL DIVIDE

MEASURE

			MOVLW		.0				;LOAD ZERO
			SUBWF		ACCcMID,1		;SUBTRACT FROM ACCcMID
		 	MOVLW		.4				;SUBTRACTION OF FACTOR  .4 for MAX
		  	SUBWF		ACCcLO,1		;SUBTRACT FROM ACCcL0
		  	BTFSS		3,0
		  	DECF		ACCcMID,1		;CHECKING FOR BORROW		  

 
M1		  BTFSC		FLAG,0				
		  GOTO		WAIT			    ;REMEASURE FLAG SET	
		  NOP							;NECESSARY FOR BANKSEL
		  BANKSEL	ADCON0				;MEASURE
		  BSF		ADCON0,ADGO			;TURN ON A/D
		  BTFSC		ADCON0,ADGO			;WAIT UNTIL FINISHED
		  GOTO		$-1
		  NOP							;NECESSARY FOR BANKSEL
 
 		  BANKSEL	PORTA				;CHANGE BANK
		  MOVF		ACCcMID,0			;CHECK IF BELOW
		  BANKSEL	ADRESH				;CHANGE BANK
		  SUBWF		ADRESH,0
		  BANKSEL	PORTA				;CHANGE BANK
		  BTFSS		3,0
		  GOTO		CLICK				;JUMP

		  NOP							;NECESSARY FOR BANKSEL	
		  BANKSEL	ADRESH				;CHANGE BANK
		  MOVF		ADRESH,0			;CHECK IF ABOVE
		  BANKSEL	PORTA
		  SUBWF		ACCcMID,0
		  BTFSS		3,0
		  GOTO		M1					;JUMP

	 	  NOP							;NECESSARY FOR BANKSEL
		  BANKSEL	PORTA				;CHANGE BANK
		  MOVF		ACCcLO,0
		  BANKSEL	ADRESL				;CHANGE BANK
		  SUBWF		ADRESL,0
		  BANKSEL	PORTA				;CHANGE BANK
		  BTFSS		3,0	
		  GOTO		CLICK				;ABOVE CALL CLICK
   		  GOTO		M1					;NOT ABOVE DO AGAIN

;*********************************************
CLICK     MOVLW		.3					;SET TIMING FOR CLICK
		  MOVWF		COUNTER2			;LOAD

		  
CLICK0	  BCF		OUT					;SPEAKER ON
		  CALL		BUZZ				;DELAY
		  BSF		OUT					;SPEAKER OFF
		  CALL		BUZZ				;DELAY
		  DECFSZ	COUNTER1,1			;DELAY FOR SOUND
		  GOTO		CLICK0				;DELAY FOR SOUND
		  BSF		OUT					;SEAKER OFF
		  INCF		COUNT,1				;INCREASE COUNT
CLICK1	  DECFSZ	COUNTER1,1			;DELAY
		  GOTO		$-1
		  DECFSZ	COUNTER2,1			;DELAY
		  GOTO		CLICK1
		  GOTO		M1					;JUMP
		  
ALARM	  BCF		INTCON,GIE			;TURN OFF INTERRUPT
	      MOVLW		.45					;THIS SET THE TIME FOR ALARMING
		  MOVWF		COUNTER2			;LOAD

ALARM1	  BCF		OUT					;TURN ON SPEAKER
		  CALL		BUZZ				;DELAY
		  BSF		OUT					;TURN OFF SPEAKER
		  CALL		BUZZ				;DELAY
		  DECFSZ	COUNTER1,1			;ALARM FOR 5 SECONDS
		  GOTO		ALARM1				;JUMP
		  DECFSZ	COUNTER2,1			;DELAY
		  GOTO		ALARM1				;REPEAT
		  BSF		OUT				    ;TURN OFF SPEAKER
		  BSF		INTCON,GIE			;TURN ON INTERRUPT
		  RETURN						;RETURN

BUZZ	  MOVLW		.4					;DELAYS
		  MOVWF		COUNTER5
		  MOVLW		.79
		  MOVWF		COUNTER4
B1		  DECFSZ	COUNTER4,1
		  GOTO		B1
		  MOVLW		.79
		  MOVWF		COUNTER4		  
		  DECFSZ	COUNTER5,1
		  GOTO		B1
		  RETURN
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Division ( 56 / 24 -> 24 )                                         -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCc, ACCd                                        -*
;*-                                                                        -*
;*-     56-bit dividend in ACCb,ACCc ( ACCb has msb's and ACCc has lsb's)  -*
;*-     24-bit divisor in ACCa                                             -*
;*-     quotient is stored in ACCc                                         -*
;*-     remainder is stored in ACCb                                        -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Div24
        call    Dsetup

dloop   bcf     STATUS,C
        rlf     ACCcLO,1         ;Rotate dividend left 1 bit position
        rlf     ACCcMID,1
        rlf     ACCcHI,1
        rlf     ACCbLO,1
        rlf     ACCbMID,1
        rlf     ACCbHI,1

        btfsc   STATUS,C        ;invert carry and exclusive or with the
        goto    clear           ;msb of the divisor then move this bit
        btfss   ACCaHI,msb_bit  ;into the lsb of the dividend
        incf    ACCcLO,1
        goto    cont
clear   btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1

cont    btfsc   ACCcLO,lsb_bit  ;check the lsb of the dividend
        goto    minus
        call    Add24           ;if = 0, then add divisor to upper 24 bits
        goto    check           ;of dividend
minus   call    Sub24           ;if = 1, then subtract divisor from upper
                                ;24 bits of dividend

check   decfsz  temp,f          ;do 24 times
        goto    dloop

        bcf     STATUS,C
        rlf     ACCcLO,1          ;shift lower 24 bits of dividend 1 bit 
        rlf     ACCcMID,1         ;position left
        rlf     ACCcHI,1
        btfsc   ACCbHI,msb_bit  ;exlusive or the inverse of the msb of the
        goto    w1              ;dividend with the msb of the divisor
        btfss   ACCaHI,msb_bit  ;store in the lsb of the dividend
        incf    ACCcLO,1
        goto    wzd
w1      btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1
wzd     btfss   ACCbHI,msb_bit  ;if the msb of the remainder is set and
        goto    wend
        btfsc   ACCaHI,msb_bit  ;the msb of the divisor is not
        goto    wend
        call    Add24           ;add the divisor to the remainder to correct
                                ;for zero partial remainder

wend    retlw   0               ;quotient in 24 lsb's of dividend
                                ;remainder in 24 msb's of dividend

Dsetup        
        movlw   0x18            ;loop 24 times
        movwf   temp
        
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     2's Complement                                                     -*
;*-                                                                        -*
;*-     Uses ACCa                                                          -*
;*-                                                                        -*
;*-     Performs 2's complement conversion on ACCa                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
compA    
        comf    ACCaLO,1          ;invert all bits in accumulator a
        comf    ACCaMID,1
        comf    ACCaHI,1
        incf    ACCaLO,1          ;add one to accumulator a
        btfsc   STATUS,Z
        incf    ACCaMID,1
        btfsc   STATUS,Z
        incf    ACCaHI,1
        retlw   0		  

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     24-bit Addition                                                    -*
;*-                                                                        -*
;*-     Uses ACCa and ACCb                                                 -*
;*-                                                                        -*
;*-     ACCa + ACCb -> ACCb                                                -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Add24
        movf    ACCaLO,W
        addwf   ACCbLO,1          ;add low bytes
        btfsc   STATUS,C        ;add in carry if necessary
        goto    A2
A1      movf    ACCaMID,W
        addwf   ACCbMID,1         ;add mid bytes
        btfsc   STATUS,C        ;add in carry if necessary
        incf    ACCbHI,1
        movf    ACCaHI,W
        addwf   ACCbHI,1          ;add high bytes
        retlw   0
A2      incf    ACCbMID,1
        btfsc   STATUS,Z
        incf    ACCbHI,1
        goto    A1
;****************************************************************************
;*--------------------------------------------------------------------------*       
;*-     Subtraction ( 24 - 24 -> 24 )                                      -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCd                                              -*
;*-                                                                        -*
;*-     ACCa -> ACCd,                                                      -*
;*-     2's complement ACCa,                                               -*
;*-     call Add24 ( ACCa + ACCb -> ACCb ),                                -*
;*-     ACCd -> ACCa                                                       -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Sub24
        movf    ACCaHI,W                ;Transfer ACCa to ACCd
        movwf   ACCdHI
        movf    ACCaMID,W
        movwf   ACCdMID
        movf    ACCaLO,W
        movwf   ACCdLO
        call    compA                   ;2's complement ACCa
        call    Add24                   ;Add ACCa to ACCb
        movf    ACCdHI,W                ;Transfer ACCd to ACCa
        movwf   ACCaHI
        movf    ACCdMID,W
        movwf   ACCaMID
        movf    ACCdLO,W
        movwf   ACCaLO
        retlw   0


		END                       ; directive 'end of program'    
    
    
    
    
    
    



